/************************************************************************************************
Sender Standheizung mit Handy steuern                                                           *
control car heater with Siemens Mobile phone.                                                   *
                                                                                                *
Bentigte Hardware:                                                                             *
Needed Hardware:                                                                                *
    AT Mega 8L                                                                                  *
	Siemens M35I                    ( RXD PD0 PIN 3, TXD PD1 PIN 4 )                            *
    Standheizung Transmitter VW T91 ( OFF PD6 PIN 12, ON PD7 PIN 13 )                           *
	Standheizung Status LED         ( PB0 PIN 14 )                                              *
    Quarz 7,3782 MHZ                ( PB6 PIN 9, PB7 PIN 10 )                                   *
                                                                                                *
                                     GER                                                        *
-----------------------------------------------------------------------------------------------	*
Der uC wird mit dem Akku vom Handy versorgt, daher kein Pegelwandler notwendig.                 *
                                                                                                *
Nach dem Einstecken werden alle SMS gelscht !                                                  *
Es darf kein PIN im Handy vergeben sein.                                                        *
                                                                                                *
Die SMS werden alle X Minuten ausgewertet.                                                      *
                                                                                                *
Nach Erhalt einer SMS mit dem Inhalt "1" wird ein Impuls an Heizung Ein gegeben.                *
Nach Erhalt einer SMS mit dem Inhalt "0" wird ein Impuls an Heizung Aus gegeben.                *
                                                                                                *
                                    EN                                                          *
-----------------------------------------------------------------------------------------------	*
The uC is Power supplied by the mobilephone -> no levechanger needed                            *
                                                                                                *
After plug in the phone all SMS will be deleted!                                                *
It must not cast a PIN in phone.                                                                *
                                                                                                *
All X minutes SMS will be checked.                                                              *
                                                                                                *
After receive a SMS with content "1" will give a pulse to heating on.                           *
After receive a SMS with content "0" will give a pulse to heating off.                          *
                                                                                                *
Last Change: 12 / 2011                                                                          *
************************************************************************************************/

#ifndef F_CPU
#define F_CPU 7378200
#endif
#define UART_BAUD 19200
#define UART_BAUD_CALC(UART_BAUD,F_OSC) ((F_CPU)/((UART_BAUD)*16L)-1)

#include <avr/io.h>           // Definition der Ein- und Ausgnge
#include <avr/interrupt.h>   // Interrupt ermglichen

#include <util/delay.h>      // einfache kurze Verzgerungsschleifen
#include <stdint.h>         // erweiterte Variablen definieren
// #include <inttypes.h>
#include <avr/sleep.h>		// Schlafmodi ansprechen
#include <avr/wdt.h>       // WatchDogTimer

#include <stdio.h>
#include <stdlib.h>
#include <string.h>


#define UART_MAXSTRLEN_0        20 // max length standard Uart
#define UART_MAXSTRLEN_1       200 // max length SMS Uart

#define CALL_WAIT_TIME        1200 // time from call begin to hang up in 10ms
#define SMS_DELETE_WAIT_TIME  2000 // SMS lschen max. Wartezeit in 100ms
#define SMS_MEMORY              20 // number of SMS Memorys
#define SMS_MAX_WAIT_TIME     5000 // max waittime for OK

#define HANDY_CHECK_SLEEP_TIME 255 // time interval in ca. 8 sec 
#define SMS_CHECK_SLEEP_TIME     7 // time interval in ca. 8 sec

#define HEATER_SWITCH_TIME    2000 // heater buton pish time in 1ms
#define MIN_RESPONSE_TIME     1800 // min. response time in 1ms


volatile uint8_t uart_str_complete_0 = 0;     // 1 .. String komplett empfangen
volatile uint8_t uart_str_count_0 = 0;
volatile char uart_string_0[UART_MAXSTRLEN_0 + 1];
volatile uint8_t uart_str_complete_1 = 0;     // 2 .. String komplett empfangen
volatile uint8_t uart_str_count_1 = 0;
volatile char uart_string_1[UART_MAXSTRLEN_1 + 1];

volatile unsigned char handynr[30];
volatile uint8_t awnser_counter = 0;


// ********************************************************************************************************************************************************************


ISR(INT1_vect)
{
  if (awnser_counter != 1)
  {
    uint16_t timecount = 0;
    for (; (timecount < MIN_RESPONSE_TIME) && (PIND & 1<<PD3); timecount++)
    {
      _delay_ms(1);
    }
    if (timecount == MIN_RESPONSE_TIME)
      awnser_counter = 1;
  }

}


ISR(WDT_vect)
{
  WDTCSR = (1<<WDIE);
}

ISR(USART_RX_vect)
{
  unsigned char nextChar;

  nextChar = UDR0;                  // read Data from Puffer
  if( uart_str_complete_0 == 0 )  	// wenn uart_string_0 gerade in Verwendung, neues Zeichen in uart_string_1 schreiben
  {
    // Daten werden erst in uart_string_0 geschrieben,
    // wenn nicht String-Ende/max Zeichenlnge erreicht ist/string gerade verarbeitet wird
    if( nextChar != '\n' &&
        nextChar != 0x0A &&
        nextChar != 0x0D &&
        uart_str_count_0 < UART_MAXSTRLEN_0 - 1 )
    {
      uart_string_0[uart_str_count_0] = nextChar;
      uart_str_count_0++;
    }
    else
    {
      if (uart_str_count_0 != 0)        // skip if no content -> only Enter
      {
        uart_string_0[uart_str_count_0] = '\0';
        uart_str_count_0 = 0;
        uart_str_complete_0 = 1;
      }
    }
  }
  else if( uart_str_complete_1 == 0 )  	// if uart_string_1 just in use, clear new char
  {
    if( nextChar != '\n' &&
        nextChar != 0x0A &&
        nextChar != 0x0D &&
        uart_str_count_1 < UART_MAXSTRLEN_1 - 1 )
    {
      uart_string_1[uart_str_count_1] = nextChar;
      uart_str_count_1++;
    }
    else
    {
      if (uart_str_count_1 != 0)        // skip if no content -> only Enter
      {
        uart_string_1[uart_str_count_1] = '\0';
        uart_str_count_1 = 0;
        uart_str_complete_1 = 1;
      }
    }
  }

}

// ********************************************************************************************************************************************************************




void init_ports()
{
  DDRD  = (1<<6) | (1<<7);	 // set ports
  DDRB  = (1<<0);
}


void final_ports() 				 // set outputs to 0
{
  DDRB  = 0;
  DDRD  = 0;
  PORTB = 0;
  PORTD = 0;
}

void init_uart()
{
  PRR &= ~(1<<PRUSART0);
  cli();
  UBRR0H=(unsigned char)(UART_BAUD_CALC(UART_BAUD,F_CPU)>>8);
  UBRR0L=(unsigned char)UART_BAUD_CALC(UART_BAUD,F_CPU);

  UCSR0B |= (1<<RXEN0) | (1<<TXEN0) | (1<<RXCIE0);  // UART RX, TX and RX Interrupt enable
  UCSR0C = (1<<UCSZ01) | (1<<UCSZ00); // Asynchron 8N1                                                                   (1<<URSEL) | fehlt

  // flush Receive-Buffer (entfernen evtl. vorhandener, ungltiger Werte)
  do
  {
    uint8_t dummy;
    (void) (dummy = UDR0);
  }
  while (UCSR0A & (1 << RXC0));
  sei();
}


void final_uart()
{
  while (!(UCSR0A & (1 << TXC0)))  // Uart send complete
    ((void)0);

  long_delay(13);
  PRR |= (1<<PRUSART0); //UART Off

  cli();
  UCSR0B = 0;
  UCSR0C = 0;
  sei();
}


void init_interrupt()
{
  EICRA = (1<<ISC11) | (1<<ISC10);	// interrupt on rising edge
  EIMSK = (1<<INT1);     // external interrupt enable
  awnser_counter = 0;
}


void final_interrupt()
{
  EIMSK = 0;     //  all external interrupts disable
}

void init_power_save()
{
  ACSR  = (1<<ACD);			 // switsh analogcomparator off
  PRR = (1<<PRTWI) | (1<<PRTIM0) | (1<<PRTIM1) | (1<<PRTIM2) | (1<<PRSPI) | (1<<PRADC); //switsh rest off
}

// ********************************************************************************************************************************************************************

int uart_putc (const uint8_t c)
{
  while (!(UCSR0A & (1 << UDRE0)));  // warten, bis UDR bereit ist fr einen neuen Wert
  UDR0 = c;  // UDR schreiben startet die bertragung
  return 1;
}


void uart_puts (const char *s)
{
  while (*s)
  {
    // so lange *s != '\0' also ungleich dem "String-Endezeichen(Terminator)"
    uart_putc(*s);
    s++;
  }
}


void long_delay(uint16_t ms)
{
  for (; ms>0; ms--) _delay_ms(1);
}


void e_long_delay(uint16_t ms)
{
  for(; ms>0; ms--) _delay_ms(10);
}


int intelli_wait(uint16_t maxwaittime, uint8_t Uartselect, char exitword[20] )// Max wait time in ms, 0 = Uart0 1 = Uart1, 0 or awaited answer
{
  int for_exit = 0;
  if (Uartselect == 0)
  {
    for (uint16_t i = 0; (i < maxwaittime) && (for_exit !=1); i++)
    {
      long_delay(1);
      if (uart_str_complete_0 == 1
          &&
          (
            exitword == 0
            ||
            strcmp (uart_string_0 , exitword) == 0
          ))
        for_exit = 1;
    }
  }
  else
  {
    for (uint16_t i = 0; (i < maxwaittime) && (for_exit !=1); i++)
    {
      long_delay(1);
      if (uart_str_complete_1 == 1
          &&
          (
            exitword == 0
            ||
            strcmp (uart_string_1 , exitword) == 0
          ))
        for_exit = 1;
    }
  }
  return for_exit;
}


void long_sleep(uint8_t sleep_count)
{
  final_uart();
  final_ports();
  set_sleep_mode(SLEEP_MODE_PWR_DOWN);
  wdt_enable(WDTO_8S);
  WDTCSR = (1<<WDIE);
  sleep_enable();
  for (; sleep_count > 0; sleep_count-- )
  {
    sleep_cpu();
  }
  sleep_disable();
  wdt_disable();
  wdt_reset();
  DDRB |= ( 1 << PB0 );  // switch PB0 LED Y ON
  cli();
  init_uart();
  sei();
}


int htoi( const char *s )
{
  return (int)strtol( s, (char **)NULL, 16 );
}

// ********************************************************************************************************************************************************************

void sms_nummerauslesen()
{
  unsigned char i;
  uint8_t for_exit = 0;
  uint64_t temp_1= 0;
  //char Buffer[20];

  strncpy(handynr, uart_string_1 , 2);  // how long is SMC Number ?
  temp_1 = htoi(handynr);
  temp_1 = (temp_1 * 2) +4; // zzgl. MSG Flags etc.

  //uart_puts("Laenge SMSC\r");
  //uart_puts(itoa(temp_1,Buffer,10)\r);

  strncpy(handynr, uart_string_1 + temp_1, 2); // read SMS number lenght

  //uart_puts("Laenge SMS Nr.\r");
  //uart_puts(handynr);

  strncpy(handynr, uart_string_1 + temp_1+4, htoi(handynr)+1); // read SMS number

  //uart_puts("Handy Nr.unbearbeitet\r");
  //uart_puts(handynr);

  for (i = 0; (i < 20) && (for_exit !=1); i++)   // count output inverted 9451--> 4915
  {
    i++;
    if(handynr[i] != 'F')
    {
      uart_putc(handynr[i]);
    }
    else
    {
      for_exit = 1;
    }
    i--;
    if(handynr[i] != 'F')
    {
      uart_putc(handynr[i]);
    }
    else
    {
      for_exit = 1;
    }
    i++;
  }
}


void fone_call ()
{
  uart_puts("atd+");
  sms_nummerauslesen();
  uart_puts(";\r");
  e_long_delay(CALL_WAIT_TIME);
  uart_puts("AT+CHUP\r");	  // hang up
}


void delete_all_sms()
{

  for( int sms_count	= 1; sms_count<SMS_MEMORY+1 ; sms_count++)
  {
    uart_str_complete_0 = 0;
    char s[3];

    uart_puts("AT+CMGD=");
    uart_puts(itoa( sms_count, s, 10 )); // send variablecontent as Ascii
    uart_putc(0x0D);
    intelli_wait(SMS_DELETE_WAIT_TIME,0,"OK");
  }
}

int heater_switch(int select_switch)
{
  //init_interrupt();
  for(uint8_t for_i = 0; for_i<2; for_i++)
  {

    switch(select_switch)
    {
    case 0:
      DDRD |= ( 1 << PB6 );
      long_delay(HEATER_SWITCH_TIME);
      DDRD &= ~( 1 << PB6 );
      //uart_puts("\rAUS\r");
      break;

    case 1:
      DDRD |= ( 1 << PB7 );
      long_delay(HEATER_SWITCH_TIME);
      DDRD &= ~( 1 << PB7 );
      //uart_puts("\rEIN\r");
      break;
    }

    long_delay(4000);
  }
  //final_interrupt();
  //if (awnser_counter ==1)
  fone_call();

  int buffer[40];
    uart_puts("\r awnser_counter =");
    uart_puts(itoa(awnser_counter,buffer,10));

}

int check_sms()
{
  uint8_t str_cut;
  uart_str_complete_0 = 0;
  uart_str_complete_1 = 0;
  uart_puts("\rAT+CMGR=1\r");
  if (intelli_wait(SMS_MAX_WAIT_TIME,1,"OK") != 1)
  {
    uart_puts("AT+CMGD= 1\r"); // delete SMS & Check SMS textcontent

    str_cut =strlen(uart_string_1) - 8;

    if (strcmp (uart_string_1 + str_cut,"03C1EA14") == 0) // AUS
      heater_switch(0);
    if (strcmp (uart_string_1 + str_cut,"03C1FA1C") == 0) // Aus
      heater_switch(0);
    if (strcmp (uart_string_1 + str_cut,"03E1FA1C") == 0) // aus
      heater_switch(0);

    if (strcmp (uart_string_1 + str_cut,"03C5A413") == 0) // EIN
      heater_switch(1);
    if (strcmp (uart_string_1 + str_cut,"03C5B41B") == 0) // Ein
      heater_switch(1);
    if (strcmp (uart_string_1 + str_cut,"03E5B41B") == 0) // ein
      heater_switch(1);

    str_cut =strlen(uart_string_1) - 4;

    if (strcmp (uart_string_1 + str_cut,"0130") == 0) // 0
      heater_switch(0);
    if (strcmp (uart_string_1 + str_cut,"0131") == 0) // 1
      heater_switch(1);
  }
  else
  {
    return 1;
  }

  return 0;
}


int handy_awnser_check()
{
  uart_str_complete_0 = 0;
  uart_puts("AT\r");
  if (intelli_wait(500,0,0) != 1)
    return 0;

  long_delay(100);
  uart_str_complete_0 =0;
  uart_puts("ATE0\r"); // disable repeat function in handy
  if (intelli_wait(500,0,"OK") == 1)
    return 1;

  uart_str_complete_0 = 0;
  uart_puts("AT\r");
  if (intelli_wait(500,0,"OK") == 1)
    return 1;

  return 0;
}



// ********************************************************************************************************************************************************************

int main(void)
{
  DDRB |= ( 1 << PB0 );  // switch PB0 LED Y ON
  init_power_save();
  init_uart();
  sei();
  long_delay(500); // wait till plug complete in jack
  while(1)
  {
    while (handy_awnser_check() == 0)
    {
      long_sleep(HANDY_CHECK_SLEEP_TIME);
    }
    delete_all_sms();
    do
    {
      long_sleep(SMS_CHECK_SLEEP_TIME);
    }
    while(check_sms() == 1 || handy_awnser_check() == 1);
  }
  return 0;
}